##############################################################################
#
# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################

from global_tools import DEBUG

from Acquisition import Implicit, aq_inner, aq_parent
from ZPublisher import BeforeTraverse

from OFS.SimpleItem import SimpleItem
from OFS.PropertyManager import PropertyManager
from Globals import InitializeClass, DTMLFile
import AccessControl

try:
    from Products.CMFCore.utils import _checkPermission
except:
    _checkPermission = None

from time import time

nobody =  AccessControl.User.nobody

ViewWhoOnline = 'View Who Online'
ViewWhoOnlineRoles = ( 'Manager','Authenticated','Anonymous')

AccessControl.Permission.registerPermissions( ( (ViewWhoOnline,(), ViewWhoOnlineRoles ) ,) )

manage_addWhoOnlineForm=DTMLFile('dtml/addWhoOnline', globals())

def manage_addWhoOnline(self, id='who_online', title='', idle = 300, REQUEST=None):
    """Add a new WhoOnline object with id *id*.
    """

    ob=WhoOnline()
    ob.id=str(id)
    ob.title=title
    ob.idle=int(idle)
    self._setObject(id, ob)
    ob=self._getOb(id)
    ob.manage_afterAdd(ob,self)
    if REQUEST is not None:
        return self.manage_main(self, REQUEST, update_menu=1)

class WhoOnline(SimpleItem,Implicit,PropertyManager):
    """ Who Online
        Tool to store who is online and who is realy active !
    """
    security =  AccessControl.ClassSecurityInfo()

    __ac_permissions__=(
                         ( ViewWhoOnline, ('getActiveMembers','getVisitors') ),
                         ( 'View', ('hasPermissions',) ),
                       )

    manage_options = (	
                         PropertyManager.manage_options
                         +SimpleItem.manage_options
                     )

    meta_type= 'Who Online'


    manage_configure = DTMLFile('dtml/configureWhoOnline', globals(), __name__='manage_configure')

    

    # add the hook with lower priotity
    def manage_afterAdd(self, item, container):
        if item is self:
            self._properties=(
                               {'id':'title', 'type': 'string', 'mode': 'w'},
                               {'id':'idle', 'type': 'int', 'mode': 'w'},
                               {'id':'ignore_manage', 'type': 'boolean' ,'mode': 'w'},
                             )
            id = self.id
            if callable(id): id = id()
            container = container.this()
            hook = BeforeTraverse.NameCaller(id)
            BeforeTraverse.registerBeforeTraverse(container, hook, self.meta_type, 99)

    # don't forget to remove the hook
    def manage_beforeDelete(self, item, container):
        if item is self:
            BeforeTraverse.unregisterBeforeTraverse(container, self.meta_type)

    def manage_editWhoOnline(self,id=None,title=None,idle=None,REQUEST=None):
        """ configuration """
        if title:
            self.title = title
        if idle:
            self.idle = int(idle)
        if self.REQUEST is not None:
            return self.manage_configure(self, REQUEST, manage_tabs_message="Properties changed.")
	
    def __init__(self):
        self.members_list = {}
        self.anons_list = {}
        self.idle = 300
        self.ignore_manage = 0
        self.ignore_extensions = ('.ico','.jpg','.gif','.png','.css','.js')
        self.ignore_ids = ('manage_main',)
        self.purge = int(time()) + 600

    # the hook ! store session data and member login if auth
    def __call__(self, client, request, response=None):
        # stack analyse to ignore some request
        ignore = 0
        stacks = request.get('TraversalRequestNameStack',None)
        if stacks is not None:
            stacks.reverse()
            for stack in stacks:
                if stack[len(stack)-1:] == '_': ignore = 1
                elif stack[:7] == 'manage_' and self.ignore_manage: ignore = 1
                elif stack in self.ignore_ids: ignore = 1
                elif stack[stack.find('.'):] in self.ignore_extensions: ignore = 1
                if ignore:
                    #DEBUG( 'Ignore request: %s' % stacks )
                    break
        stacks.reverse()
        if not ignore:

            t = int(time())
            session = request.SESSION.getId()
            try:
                #is that really ok ? not sure that is the best way to get active member
                member = request._authUserPW()[0]
            except:
                member = 'Anonymous User'

            #DEBUG( 'Request by %s. Session id:  %s' % (member,session) )

            if member == 'Anonymous User':
                self.anons_list[session] = {'time':t}
                for k in self.members_list.keys():
                    if self.members_list[k]['session'] == session:
                        #DEBUG( 'Member loggued out. Session %s' % session )
                        del self.members_list[k]
            else:
                self.members_list[member] = {'id':member,'session':session,'time':t}
                if self.anons_list.has_key(session):
                    #DEBUG( '%s loggued. Ex session %s' % (member,session) )
                    del self.anons_list[session]

    def hasPermission(self,REQUEST=None):
        ''' return 1 if user is authorised to see Who is Online '''
        REQUEST = self.REQUEST
        if REQUEST is not None:
            user = REQUEST.get('AUTHENTICATED_USER',None)
            if user is None: user = nobody
            if user.has_permission(ViewWhoOnline, self):
                return 1
        return None

    # for a non CMFSite return a list of users id
    security.declarePrivate( '_getActiveMembers' )
    def _getActiveMembers(self):
        t = int(time()) - self.idle
        online = []
        #DEBUG('Active Members: %s' % self.members_list.values())
        for m in self.members_list.values():
            if m['time'] > t:
                online.append(m['id'])
        if len(online) > 0:
            online.sort()
            return online
        return None

    # for a CMFSite return a list of member curently active
    security.declarePrivate( '_getActiveCMFMembers' )
    def _getActiveCMFMembers(self):
        t = int(time()) - self.idle
        getMemberById = None
        md = self.portal_memberdata

        online = []

        portal = self.portal_url.getPortalObject()

        isManager = _checkPermission('Manage portal', self)

        members_list = self.members_list.values()

        members_list.sort()

        #DEBUG('Active Members: %s' % self.members_list)

        for m in members_list:
            if m['time'] > t:
                user = portal.acl_users.getUser(m['id'])
                if user is not None:
                    mb = md.wrapUser(user)
                    if mb is not None and (isManager or mb.listed):
                        online.append(mb)
        return online

    def getActiveMembers(self):
        """ Return active members """
        self._purge()
        if hasattr(self,'portal_membership'): return self._getActiveCMFMembers()
        else: return self._getActiveMembers()

    # return number of active Anonymous
    def getActiveVisitors(self):
        """ Return how many Anonymous visitors are currently online """
        self._purge()
        t = int(time()) - self.idle
        online = []
        for m in self.anons_list.values():
            if m['time'] > t:
                online.append(m)
        return len(online)

    # Purge anons_list each 600s. Never purge members_list.
    security.declarePrivate( '_purge' )
    def _purge(self):
        t = int(time()) - self.idle
        if self.purge < t:
            self.purge = t + 600
            ks = self.anons_list.keys()
            for k in ks:
                # May be the key was deleted by another hook... ? It's seems bug whithout testing.
                if self.anons_list.has_key(k):
                    if self.anons_list[k]['time'] < t:
                        #DEBUG( 'Purge Anonyme with session %s' % k )
                        del self.anons_list[k]
    # non use
    security.declarePublic( 'isOnline' )
    def isOnline(self,member):
        """ Return if a member is online """
        t = int(time()) - self.idle
        if self.members_list.has_key(member):
            if self.members_list['member']['time'] > t:
                return 1
        return None

InitializeClass(WhoOnline)
